@@ -1,8 +1,16 @@
-from datetime import datetime as dt
+"""
+``Card`` object dedicated to AOI selection. It does not include maps.
+"""
 
+from datetime import datetime as dt
+from pathlib import Path
+from typing import Dict, List, Optional, Union
+
+import ipyvuetify as v
 import pandas as pd
+import traitlets as t
 from deprecated.sphinx import versionadded
-from traitlets import Int
+from typing_extensions import Self
 
 import sepal_ui.sepalwidgets as sw
 from sepal_ui import mapping as sm
@@ -20,21 +28,26 @@ __all__ = ["AoiView", "select_methods"]
 
 
 class MethodSelect(sw.Select):
-    f"""
-    A method selector. It will list the available methods for this very AoiView.
-    'ALL' will select all the available methods (default)
-    'ADMIN' only the admin one, 'CUSTOM' only the custom one.
-    'XXX' will add the selected method to the list when '-XXX' will discard it.
-    You cannot mix adding and removing behaviours.
+    def __init__(
+        self,
+        methods: Union[str, List[str]] = "ALL",
+        gee: bool = True,
+        map_: Optional[sm.SepalMap] = None,
+    ) -> None:
+        """
+        A method selector.
 
-    Args:
-        methods (str|[str]): a list of methods from the available list ({', '.join(select_methods.keys())})
-        map_ (SepalMap, optional): link the aoi_view to a custom SepalMap to display the output, default to None
-        gee (bool, optional): wether to bind to ee or not
-    """
-
-    def __init__(self, methods="ALL", gee=True, map_=None):
+        It will list the available methods for this very AoiView.
+        'ALL' will select all the available methods (default)
+        'ADMIN' only the admin one, 'CUSTOM' only the custom one.
+        'XXX' will add the selected method to the list when '-XXX' will discard it.
+        You cannot mix adding and removing behaviours.
 
+        Args:
+            methods: a list of methods from the available list (ADMIN0, ADMIN1, ADMIN2, SHAPE, DRAW, POINTS, ASSET)
+            map_: link the aoi_view to a custom SepalMap to display the output, default to None
+            gee: wether to bind to ee or not
+        """
         # create the method list
         if methods == "ALL":
             self.methods = select_methods
@@ -86,29 +99,31 @@ class MethodSelect(sw.Select):
 
 
 class AdminField(sw.Select):
-    """
-    An admin level selector. It is binded to ee (GAUL 2015) or not (GADM 2021). allows to select administrative codes taking into account the administrative parent code and displaying humanly readable administrative names.
 
-    Args:
-        level (int): The administrative level of the field
-        parent (AdminField): the adminField that deal with the parent admin level of the current selector. used to narrow down the possible options
-        ee (bool, optional): wether to use ee or not (default to True)
+    gee: bool = True
+    "wether or not to depend on earthengine"
 
-    Attributes:
-        gee (bool): the earthengine status
-        level (int): the admin level of the current field
-        parent (AdminField): the field parent object
-    """
+    level: int = -1
+    "The admin level of the current field"
 
-    # the file location of the database
-    FILE = AoiModel.FILE
-    CODE = AoiModel.CODE
-    NAME = AoiModel.NAME
+    parent: Optional[sw.Select] = None
+    "The parent adminfield object"
 
-    def __init__(self, level, parent=None, gee=True):
+    def __init__(
+        self, level: int, parent: Optional[sw.Select] = None, gee: bool = True
+    ) -> None:
+        """
+        An admin level selector.
 
+        It is binded to ee (GAUL 2015) or not (GADM 2021). allows to select administrative codes taking into account the administrative parent code and displaying humanly readable administrative names.
+
+        Args:
+            level: The administrative level of the field
+            parent: the adminField that deal with the parent admin level of the current selector. used to narrow down the possible options
+            ee: wether to use ee or not (default to True)
+        """
         # save ee state
-        self.ee = gee
+        self.gee = gee
 
         # get the level info
         self.level = level
@@ -122,57 +137,49 @@ class AdminField(sw.Select):
         # add js behaviour
         self.parent is None or self.parent.observe(self._update, "v_model")
 
-    def show(self):
+    def show(self) -> Self:
         """
-        when an admin field is shown, show its parent as well
-
-        Return:
-            self
+        when an admin field is shown, show its parent as well.
         """
-
         super().show()
         self.parent is None or self.parent.show()
 
         return self
 
-    def get_items(self, filter_=None):
+    def get_items(self, filter_: str = "") -> Self:
+        r"""
+        Update the item list based on the given filter.
+
+        Args:
+            filter\_ (str): The code of the parent v_model to filter the current results
         """
-        update the item list based on the given filter
-
-        Params:
-            filter\\_ (str): The code of the parent v_model to filter the current results
-
-        Return:
-            self
-        """
-
         # extract the level list
         df = (
-            pd.read_csv(self.FILE[self.ee])
-            .drop_duplicates(subset=self.CODE[self.ee].format(self.level))
-            .sort_values(self.NAME[self.ee].format(self.level))
+            pd.read_parquet(AoiModel.FILE[self.gee])
+            .astype(str)
+            .drop_duplicates(subset=AoiModel.CODE[self.gee].format(self.level))
+            .sort_values(AoiModel.NAME[self.gee].format(self.level))
         )
 
         # filter it
         if filter_:
-            df = df[df[self.CODE[self.ee].format(self.level - 1)] == filter_]
+            df = df[df[AoiModel.CODE[self.gee].format(self.level - 1)] == filter_]
 
         # formatted as a item list for a select component
         self.items = [
             {
                 "text": su.normalize_str(
-                    r[self.NAME[self.ee].format(self.level)], folder=False
+                    r[AoiModel.NAME[self.gee].format(self.level)], folder=False
                 ),
-                "value": r[self.CODE[self.ee].format(self.level)],
+                "value": r[AoiModel.CODE[self.gee].format(self.level)],
             }
             for _, r in df.iterrows()
         ]
 
         return self
 
-    def _update(self, change):
-        """update the item list of the admin select"""
-
+    def _update(self, change: dict) -> Self:
+        """update the item list of the admin select."""
         # reset v_model
         self.v_model = None
 
@@ -184,89 +191,100 @@ class AdminField(sw.Select):
 
 
 class AoiView(sw.Card):
-    """
-    Versatile card object to deal with the aoi selection. multiple selection method are available (see the MethodSelector object) and the widget can be fully customizable. Can also be bound to ee (ee==True) or not (ee==False)
-
-    Args:
-        methods (list, optional): the methods to use in the widget, default to 'ALL'. Available: {'ADMIN0', 'ADMIN1', 'ADMIN2', 'SHAPE', 'DRAW', 'POINTS', 'ASSET', 'ALL'}
-        map\\_ (SepalMap, optional): link the aoi_view to a custom SepalMap to display the output, default to None
-        gee (bool, optional): wether to bind to ee or not
-        vector (str|pathlib.Path, optional): the path to the default vector object
-        admin (int, optional): the administrative code of the default selection. Need to be GADM if :code:`ee==False` and GAUL 2015 if :code:`ee==True`.
-        asset (str, optional): the default asset. Can only work if :code:`ee==True`
-    """
 
     # ##########################################################################
     # ###                             widget parameters                      ###
     # ##########################################################################
 
-    updated = Int(0).tag(sync=True)
-    "int: traitlets triggered every time a AOI is selected"
+    updated: t.Int = t.Int(0).tag(sync=True)
+    "Traitlets triggered every time a AOI is selected"
 
-    ee = True
-    "bool: either or not he aoi_view is connected to gee"
+    gee: bool = True
+    "Either or not he aoi_view is connected to gee"
 
-    folder = None
-    "str: the folder name used in GEE related component, mainly used for debugging"
+    folder: Union[str, Path] = ""
+    "The folder name used in GEE related component, mainly used for debugging"
 
-    model = None
-    "sepal_ui.aoi.AoiModel: the model to create the AOI from the selected parameters"
+    model: Optional[AoiModel] = None
+    "The model to create the AOI from the selected parameters"
+
+    map_style: Optional[dict] = None
+    "The predifined style of the aoi on the map"
 
     # ##########################################################################
     # ###                            the embeded widgets                     ###
     # ##########################################################################
 
-    map_ = None
-    "sepal_ui.mapping.SepalMap: the map to draw the AOI"
+    map_: Optional[sm.SepalMap] = None
+    "The map to draw the AOI"
 
-    aoi_dc = None
-    "sepal_ui.mapping.DrawControl: the drawing control associated with DRAW method"
+    aoi_dc: Optional[sm.DrawControl] = None
+    "the drawing control associated with DRAW method"
 
-    w_method = None
-    "widget: the widget to select the method"
+    w_method: Optional[MethodSelect] = None
+    "The widget to select the method"
 
-    components = None
-    "dict: the followingwidgets used to define AOI"
+    components: Dict[str, v.VuetifyWidget] = {}
+    "The followingwidgets used to define AOI"
 
-    w_admin_0 = None
-    "widget: the widget used to select admin level 0"
+    w_admin_0: Optional[AdminField] = None
+    "The widget used to select admin level 0"
 
-    w_admin_1 = None
-    "widget: the widget used to select admin level 1"
+    w_admin_1: Optional[AdminField] = None
+    "The widget used to select admin level 1"
 
-    w_admin_2 = None
-    "widget: the widget used to select admin level 2"
+    w_admin_2: Optional[AdminField] = None
+    "The widget used to select admin level 2"
 
-    w_vector = None
-    "widget: the widget used to select vector shapes"
+    w_vector: Optional[sw.VectorField] = None
+    "The widget used to select vector shapes"
 
-    w_points = None
-    "widget: the widget used to select points files"
+    w_points: Optional[sw.LoadTableField] = None
+    "The widget used to select points files"
 
-    w_draw = None
-    "widget: the widget used to select the name of a drawn shape (only if :code:`map_ != None`)"
+    w_draw: Optional[sw.TextField] = None
+    "The widget used to select the name of a drawn shape (only if :code:`map_ != None`)"
 
-    w_asset = None
-    "widget: the widget used to select asset name of a featureCollection (only if :code:`gee == True`)"
+    w_asset: Optional[sw.AssetSelect] = None
+    "The widget used to select asset name of a featureCollection (only if :code:`gee == True`)"
 
-    btn = None
-    "sw.Btn: a default btn"
+    btn: Optional[sw.Btn] = None
+    "A default btn"
 
-    alert = None
-    "sw.Alert: a alert to display message to the end user"
+    alert: Optional[sw.Alert] = None
+    "A alert to display message to the end user"
 
     @versionadded(
         version="2.11.3",
         reason="Model is now an optional parameter to AoiView, it can be created from outside and passed to the initialization function.",
     )
     def __init__(
-        self, methods="ALL", map_=None, gee=True, folder="", model=None, **kwargs
-    ):
+        self,
+        methods: Union[str, List[str]] = "ALL",
+        map_: Optional[sm.SepalMap] = None,
+        gee: bool = True,
+        folder: Union[str, Path] = "",
+        model: Optional[AoiModel] = None,
+        map_style: Optional[dict] = None,
+        **kwargs,
+    ) -> None:
+        r"""
+        Versatile card object to deal with the aoi selection. multiple selection method are available (see the MethodSelector object) and the widget can be fully customizable. Can also be bound to ee (ee==True) or not (ee==False).
 
+        Args:
+            methods: the methods to use in the widget, default to 'ALL'. Available: {'ADMIN0', 'ADMIN1', 'ADMIN2', 'SHAPE', 'DRAW', 'POINTS', 'ASSET', 'ALL'}
+            map\_: link the aoi_view to a custom SepalMap to display the output, default to None
+            gee: wether to bind to ee or not
+            vector: the path to the default vector object
+            admin: the administrative code of the default selection. Need to be GADM if :code:`ee==False` and GAUL 2015 if :code:`ee==True`.
+            asset: the default asset. Can only work if :code:`ee==True`
+            map_style: the predifined style of the aoi. It's by default using a "success" ``sepal_ui.color`` with 0.5 transparent fill color. It can be completly replace by a fully qualified `style dictionnary <https://ipyleaflet.readthedocs.io/en/latest/layers/geo_json.html>`__. Use the ``sepal_ui.color`` object to define any color to remain compatible with light and dark theme.
+        """
         # set ee dependencie
-        self.ee = gee
+        self.gee = gee
         self.folder = folder
-        gee is False or su.init_ee()
+        if gee is True:
+            su.init_ee()
 
         # get the model
         self.model = model or AoiModel(gee=gee, folder=folder, **kwargs)
@@ -274,6 +292,9 @@ class AoiView(sw.Card):
         # get the map if filled
         self.map_ = map_
 
+        # get the aoi geoJSON style
+        self.map_style = map_style
+
         # create the method widget
         self.w_method = MethodSelect(methods, gee=gee, map_=map_)
 
@@ -312,13 +333,13 @@ class AoiView(sw.Card):
         # defint the asset select separately. If no gee is set up we don't want any
         # gee based widget to be requested. If it's the case, application that does not support GEE
         # will crash if the user didn't authenticate
-        if self.ee:
+        if self.gee:
             self.w_asset = sw.VectorField(
                 label=ms.aoi_sel.asset, gee=True, folder=self.folder, types=["TABLE"]
             )
             self.w_asset.hide()
             self.components["ASSET"] = self.w_asset
-            self.model.bind(self.w_asset, "asset_name")
+            self.model.bind(self.w_asset, "asset_json")
 
         # define DRAW option separately as it will only work if the map is set
         if self.map_:
@@ -329,7 +350,7 @@ class AoiView(sw.Card):
             self.aoi_dc.hide()
 
         # add a validation btn
-        self.btn = sw.Btn(ms.aoi_sel.btn)
+        self.btn = sw.Btn(msg=ms.aoi_sel.btn)
 
         # create the widget
         self.children = (
@@ -346,9 +367,8 @@ class AoiView(sw.Card):
         self.model.clear_attributes()
 
     @sd.loading_button(debug=True)
-    def _update_aoi(self, widget, event, data):
-        """load the object in the model & update the map (if possible)"""
-
+    def _update_aoi(self, *args) -> Self:
+        """load the object in the model & update the map (if possible)."""
         # read the information from the geojson datas
         if self.map_:
             self.model.geo_json = self.aoi_dc.to_json()
@@ -361,11 +381,7 @@ class AoiView(sw.Card):
         if self.map_:
             self.map_.remove_layer("aoi", none_ok=True)
             self.map_.zoom_bounds(self.model.total_bounds())
-
-            if self.ee:
-                self.map_.add_ee_layer(self.model.feature_collection, {}, name="aoi")
-            else:
-                self.map_.add_layer(self.model.get_ipygeojson())
+            self.map_.add_layer(self.model.get_ipygeojson(self.map_style))
 
             self.aoi_dc.hide()
 
@@ -374,21 +390,20 @@ class AoiView(sw.Card):
 
         return self
 
-    def reset(self):
-        """clear the aoi_model from input and remove the layer from the map (if existing)"""
-
+    def reset(self) -> Self:
+        """clear the aoi_model from input and remove the layer from the map (if existing)."""
         # reset the view of the widgets
         self.w_method.v_model = None
 
         # clear the map
-        self.map_ is None or self.map_.remove_layer("aoi", none_ok=True)
+        if self.map_ is not None:
+            self.map_.remove_layer("aoi", none_ok=True)
 
         return self
 
     @sd.switch("loading", on_widgets=["w_method"])
-    def _activate(self, change):
-        """activate the adapted widgets"""
-
+    def _activate(self, change: dict) -> None:
+        """activate the adapted widgets."""
         # clear and hide the alert
         self.alert.reset()
 
