@@ -1,88 +1,90 @@
+"""
+Customized drawing control.
+"""
+
 from copy import deepcopy
+from typing import Optional
 
 import geopandas as gpd
-from ipyleaflet import DrawControl
+import ipyleaflet as ipl
 from shapely import geometry as sg
 
 from sepal_ui import color
 
 
-class DrawControl(DrawControl):
-    """
-    A custom DrawingControl object to handle edition of features
+class DrawControl(ipl.DrawControl):
 
-    Args:
-        m (ipyleaflet.Map): the map on which he drawControl is displayed
-        kwargs (optional): any available arguments from a ipyleaflet.DrawingControl
-    """
+    m: Optional[ipl.Map] = None
+    "the map on which he drawControl is displayed. It will help control the visibility"
 
-    m = None
-    "(ipyleaflet.Map) the map on which he drawControl is displayed. It will help control the visibility"
-
-    def __init__(self, m, **kwargs):
+    def __init__(self, m: ipl.Map, **kwargs) -> None:
+        """
+        A custom DrawingControl object to handle edition of features.
 
+        Args:
+            m: the map on which he drawControl is displayed
+            kwargs: any available arguments from a ipyleaflet.DrawingControl
+        """
         # set some default parameters
         options = {"shapeOptions": {"color": color.info}}
-        kwargs["marker"] = kwargs.pop("marker", {})
-        kwargs["circlemarker"] = kwargs.pop("circlemarker", {})
-        kwargs["polyline"] = kwargs.pop("polyline", {})
-        kwargs["rectangle"] = kwargs.pop("rectangle", options)
-        kwargs["circle"] = kwargs.pop("circle", options)
-        kwargs["polygon"] = kwargs.pop("polygon", options)
+        kwargs.setdefault("marker", {})
+        kwargs.setdefault("circlemarker", {})
+        kwargs.setdefault("polyline", {})
+        kwargs.setdefault("rectangle", options)
+        kwargs.setdefault("circle", options)
+        kwargs.setdefault("polygon", options)
 
         # save the map in the member of the objects
         self.m = m
 
         super().__init__(**kwargs)
 
-    def show(self):
+    def show(self) -> None:
         """
         show the drawing control on the map. and clear it's content.
         """
-
         self.clear()
         self in self.m.controls or self.m.add(self)
 
         return
 
-    def hide(self):
+    def hide(self) -> None:
         """
         hide the drawing control from the map, and clear it's content.
         """
-
         self.clear()
         self not in self.m.controls or self.m.remove(self)
 
         return
 
-    def to_json(self):
+    def to_json(self) -> dict:
         """
-        Return the content of the DrawCOntrol data without the styling properties and using a polygonized representation of circles.
-        The output is fully compatible with __geo_interface__.
+        Return the content of the DrawControl data.
 
-        Return:
-            (dict): the json representation of all the geometries draw on the map
+        Returned without the styling properties and using a polygonized representation of circles. The output is fully compatible with __geo_interface__.
+
+        Returns:
+            the json representation of all the geometries draw on the map
         """
-
         features = [self.polygonize(feat) for feat in deepcopy(self.data)]
         [feat["properties"].pop("style") for feat in features]
 
         return {"type": "FeatureCollection", "features": features}
 
     @staticmethod
-    def polygonize(geo_json):
+    def polygonize(geo_json: dict) -> dict:
         """
         Transform a ipyleaflet circle (a point with a radius) into a GeoJson polygon.
+
         The methods preserves all the geo_json other attributes.
         If the geometry is not a circle (don't require polygonisation), do nothing.
 
         Params:
-            geo_json (json): the circle geojson
+            geo_json: the circle geojson
 
-        Return:
-            (dict): the polygonised feature
+        Returns:
+            the polygonised feature
         """
-
         if "Point" not in geo_json["geometry"]["type"]:
             return geo_json
 
