@@ -1,3 +1,7 @@
+"""
+All the helper function of sepal-ui. 
+"""
+
 import math
 import os
 import random
@@ -5,11 +9,12 @@ import re
 import string
 import warnings
 from pathlib import Path
+from typing import Any, Sequence, Union
 from urllib.parse import urlparse
 
 import ee
 import httplib2
-from cryptography.fernet import Fernet
+import ipyvuetify as v
 from deprecated.sphinx import deprecated, versionadded
 from matplotlib import colors as c
 from unidecode import unidecode
@@ -21,14 +26,16 @@ from sepal_ui.scripts import decorator as sd
 from sepal_ui.scripts.warning import SepalWarning
 
 
-def hide_component(widget):
+def hide_component(widget: v.VuetifyWidget) -> v.VuetifyWidget:
     """
-    hide a vuetify based component
+    hide a vuetify based component.
 
     Args:
-        widget (v.VuetifyWidget): the widget to hide
-    """
+        widget: the widget to hide
 
+    Returns:
+        the widget
+    """
     if isinstance(widget, sepal_ui.sepalwidgets.sepalwidget.SepalWidget):
         widget.hide()
 
@@ -38,14 +45,16 @@ def hide_component(widget):
     return widget
 
 
-def show_component(widget):
+def show_component(widget: v.VuetifyWidget) -> v.VuetifyWidget:
     """
-    show a vuetify based component
+    show a vuetify based component.
 
     Args:
-        widget (v.VuetifyWidget): the widget to hide
-    """
+        widget: the widget to hide
 
+    Returns:
+        the widget
+    """
     if isinstance(widget, sepal_ui.sepalwidgets.sepalwidget.SepalWidget):
         widget.show()
 
@@ -55,17 +64,16 @@ def show_component(widget):
     return widget
 
 
-def create_download_link(pathname):
+def create_download_link(pathname: Union[str, Path]) -> str:
     """
-    Create a clickable link to download the pathname target
+    Create a clickable link to download the pathname target.
 
     Args:
-        pathname (str | pathlib.Path): the pathname th download
+        pathname: the pathname th download
 
-    Return:
-        (str): the download link
+    Returns:
+        the download link
     """
-
     # return the link if it's an absolute url
     if isinstance(pathname, str) and bool(urlparse(str(pathname)).netloc):
         return pathname
@@ -85,33 +93,31 @@ def create_download_link(pathname):
     return link
 
 
-def random_string(string_length=3):
+def random_string(string_length: int = 3) -> str:
     """
     Generates a random string of fixed length.
 
     Args:
-        string_length (int, optional): Fixed length. Defaults to 3.
+        string_length: Fixed length. Defaults to 3.
 
-    Return:
-        (str): A random string
+    Returns:
+        A random string
     """
-
     letters = string.ascii_lowercase
 
     return "".join(random.choice(letters) for i in range(string_length))
 
 
-def get_file_size(filename):
+def get_file_size(filename: Union[str, Path]) -> str:
     """
-    Get the file size as string of 2 digit in the adapted scale (B, KB, MB....)
+    Get the file size as string of 2 digit in the adapted scale (B, KB, MB....).
 
     Args:
-        filename (str | pathlib.Path): the path to the file to mesure
+        filename: the path to the file to mesure
 
-    Return:
-        (str): the file size in a readable humanly readable
+    Returns:
+        the file size in a readable humanly readable
     """
-
     file_size = Path(filename).stat().st_size
 
     if file_size == 0:
@@ -125,79 +131,65 @@ def get_file_size(filename):
     return "{:.1f} {}".format(s, size_name[i])
 
 
-def init_ee():
+def init_ee() -> None:
     """
     Initialize earth engine according to the environment.
-    It will use the creddential file if the EE_PRIVATE_KEY env variable exist.
-    Otherwise it use the simple Initialize command (asking the user to register if necessary)
-    """
 
+    It will use the creddential file if the EARTHENGINE_TOKEN env variable exist.
+    Otherwise it use the simple Initialize command (asking the user to register if necessary).
+    """
     # only do the initialization if the credential are missing
     if not ee.data._credentials:
 
-        # if the decrypt key is available use the decript key
-        if "EE_DECRYPT_KEY" in os.environ:
+        # if the credentials token is asved in the environment use it
+        if "EARTHENGINE_TOKEN" in os.environ:
 
-            # read the key as byte
-            key = os.environ["EE_DECRYPT_KEY"].encode()
+            # write the token to the appropriate folder
+            ee_token = os.environ["EARTHENGINE_TOKEN"]
+            credential_folder_path = Path.home() / ".config" / "earthengine"
+            credential_folder_path.mkdir(parents=True, exist_ok=True)
+            credential_file_path = credential_folder_path / "credentials"
+            credential_file_path.write_text(ee_token)
 
-            # create the fernet object
-            fernet = Fernet(key)
-
-            # decrypt the key
-            json_encrypted = Path(__file__).parent / "encrypted_key.json"
-            with json_encrypted.open("rb") as f:
-                json_decripted = fernet.decrypt(f.read()).decode()
-
-            # write it to a file
-            with open("ee_private_key.json", "w") as f:
-                f.write(json_decripted)
-
-            # connection to the service account
-            service_account = "test-sepal-ui@sepal-ui.iam.gserviceaccount.com"
-            credentials = ee.ServiceAccountCredentials(
-                service_account, "ee_private_key.json"
-            )
-            ee.Initialize(credentials, http_transport=httplib2.Http())
-
-        # if in local env use the local user credential
-        else:
-            ee.Initialize(http_transport=httplib2.Http())
+        # if the user is in local development the authentication should
+        # already be available
+        ee.Initialize(http_transport=httplib2.Http())
 
     return
 
 
-def normalize_str(msg, folder=True):
+def normalize_str(msg: str, folder: bool = True) -> str:
     """
-    Normalize an str to make it compatible with file naming (no spaces, special chars ...etc)
+    Normalize an str to make it compatible with file naming (no spaces, special chars ...etc).
 
     Params:
-        msg (str): the string to sanitise
-        folder (optional|bool): if the name will be used for folder naming or for display. if display, <'> and < > characters will be kept
+        msg: the string to sanitise
+        folder: if the name will be used for folder naming or for display. if display, <'> and < > characters will be kept
 
-    Return:
-        (str): the modified str
+    Returns:
+        the modified str
     """
-
     regex = "[^a-zA-Z\d\-\_]" if folder else "[^a-zA-Z\d\-\_\ ']"
 
     return re.sub(regex, "_", unidecode(msg))
 
 
-def to_colors(in_color, out_type="hex"):
+def to_colors(
+    in_color: Union[str, Sequence], out_type: str = "hex"
+) -> Union[str, tuple]:
     """
-    Transform any color type into a color in the specified output format
-    avalable format: hex
+    Transform any color type into a color in the specified output format.
+
+    Avalable format: [hex]
 
     Args:
-        in_color (str or tuple): It can be a string (e.g., 'red', '#ffff00', 'ffff00') or RGB tuple (e.g., (255, 127, 0)).
-        out_type (str, optional): the type of the output color from ['hex']. default to 'hex'
+        in_color: It can be a string (e.g., 'red', '#ffff00', 'ffff00') or RGB tuple (e.g., (255, 127, 0)).
+        out_type: the type of the output color from ['hex']. default to 'hex'
 
     Returns:
-        (str|tuple): The color in the specified format. default to black.
+        The color in the specified format. default to black.
     """
-
-    # list of the color function used for the translatio
+    # list of the color function used for the translation
     c_func = {"hex": c.to_hex}
     transform = c_func[out_type]
 
@@ -228,17 +220,16 @@ def to_colors(in_color, out_type="hex"):
     return transform(out_color)
 
 
-def next_string(string):
+def next_string(string: str) -> str:
     """
-    Create a string followed by an underscore and a consecutive number
+    Create a string followed by an underscore and a consecutive number.
 
     Args:
-        string (str): the initial string
+        string: the initial string
 
     Returns:
-        (str): the incremented string
+        the incremented string
     """
-
     # if the string is already numbered the last digit is separeted from the rest of the string by an "_"
     split = string.split("_")
     end = split[-1]
@@ -251,17 +242,18 @@ def next_string(string):
     return string
 
 
-def set_config(key, value, section="sepal-ui"):
+def set_config(key: str, value: str, section: str = "sepal-ui") -> None:
     """
+    Set a config variable.
+
     Set the provided value to the given key for the given section in the sepal-ui config
-    file
+    file.
 
     Args:
-        key (str): key configuration name
-        value (str): value to be referenced by the configuration key
-        section (str, optional): configuration section, defaults to sepal-ui.
+        key: key configuration name
+        value: value to be referenced by the configuration key
+        section: configuration section, defaults to sepal-ui.
     """
-
     # set the section if needed
     if "sepal-ui" not in config.sections():
         config.add_section(section)
@@ -278,42 +270,41 @@ def set_config(key, value, section="sepal-ui"):
 @deprecated(
     version="2.9.1", reason="This function will be removed in favor of set_config()"
 )
-def set_config_locale(locale):
+def set_config_locale(locale: str) -> None:
     """
-    Set the provided local in the sepal-ui config file
+    Set the provided local in the sepal-ui config file.
 
     Args:
-        locale (str): a locale name in IETF BCP 47 (no verifications are performed)
+        locale: a locale name in IETF BCP 47 (no verifications are performed)
     """
-
     return set_config("locale", locale)
 
 
 @deprecated(
     version="2.9.1", reason="This function will be removed in favor of set_config()"
 )
-def set_config_theme(theme):
+def set_config_theme(theme: str) -> None:
     """
-    Set the provided theme in the sepal-ui config file
+    Set the provided theme in the sepal-ui config file.
 
     Args:
-        theme (str): a theme name (currently supporting "dark" and "light")
+        theme: a theme name (currently supporting "dark" and "light")
     """
-
     return set_config("theme", theme)
 
 
 @versionadded(version="2.7.1")
-def set_type(color):
-    """
-    Return a pre-defined material colors based on the requested type\_ parameter. If the parameter is not a predifined color,
-    fallback to "info" and will raise a warning. the colors can only be selected from ["primary", "secondary", "accent", "error", "info", "success", "warning", "anchor"]
+def set_type(color: str) -> str:
+    r"""
+    Return a pre-defined material colors based on the requested type\_ parameter.
+
+    If the parameter is not a predifined color, fallback to "info" and will raise a warning. the colors can only be selected from ["primary", "secondary", "accent", "error", "info", "success", "warning", "anchor"].
 
     Args:
-        color (str): the requested color
+        color: the requested color
 
     Returns:
-        (str): a pre-defined material color
+        a pre-defined material color
 
     """
     from sepal_ui.frontend.styles import TYPES
@@ -329,21 +320,23 @@ def set_type(color):
 
 
 @versionadded(version="2.8.0")
-def geojson_to_ee(geo_json, geodesic=False, encoding="utf-8"):
+def geojson_to_ee(
+    geo_json: dict, geodesic: bool = False, encoding: str = "utf-8"
+) -> ee.FeatureCollection:
     """
-    Transform a geojson object into a featureCollection or a Geometry
+    Transform a geojson object into a featureCollection or a Geometry.
+
     No sanity check is performed on the initial geo_json. It must respect the
     `__geo_interface__ <https://gist.github.com/sgillies/2217756>`__.
 
     Args:
-        geo_json (dict): a geo_json dictionnary
-        geodesic (bool, optional): Whether line segments should be interpreted as spherical geodesics. If false, indicates that line segments should be interpreted as planar lines in the specified CRS. If absent, defaults to True if the CRS is geographic (including the default EPSG:4326), or to False if the CRS is projected. Defaults to False.
-        encoding (str, optional): The encoding of characters. Defaults to "utf-8".
+        geo_json: a geo_json dictionnary
+        geodesic: Whether line segments should be interpreted as spherical geodesics. If false, indicates that line segments should be interpreted as planar lines in the specified CRS. If absent, defaults to True if the CRS is geographic (including the default EPSG:4326), or to False if the CRS is projected. Defaults to False.
+        encoding: The encoding of characters. Defaults to "utf-8".
 
     Returns:
-        (ee.FeatureCollection): the created featurecollection
+        the created featurecollection
     """
-
     # from a featureCollection
     if geo_json["type"] == "FeatureCollection":
         for feature in geo_json["features"]:
@@ -374,19 +367,19 @@ def geojson_to_ee(geo_json, geodesic=False, encoding="utf-8"):
     return
 
 
-def check_input(input_, msg=ms.utils.check_input.error):
-    """
+def check_input(input_: Any, msg: str = ms.utils.check_input.error) -> bool:
+    r"""
     Check if the inpupt value is initialized.
-    If not raise an error, else return True
+
+    If not raise an error, else return True.
 
     Args:
-        input\_ (any): the input to check
-        msg (str, optionnal): the message to display if the input is not set
+        input\_: the input to check
+        msg: the message to display if the input is not set
 
     Return:
-        (bool): check if the value is initialized
+        check if the value is initialized
     """
-
     # by the default the variable is considered valid
     init = True
 
