@@ -1,21 +1,40 @@
+"""
+Custom widgets relative to user application framework.
+
+Gather the customized ``ipyvuetifyWidgets`` used to create the application framework.
+All the content of this modules is included in the parent ``sepal_ui.sepalwidgets`` package. So it can be imported directly from there.
+
+Example:
+    .. jupyter-execute::
+    
+        from sepal_ui import sepalwidgets as sw
+        
+        sw.LocaleSelect()
+"""
+
 from datetime import datetime
 from functools import partial
 from itertools import cycle
 from pathlib import Path
+from typing import Dict, List, Optional, Union
 
 import ipyvuetify as v
 import pandas as pd
+import traitlets as t
 from deprecated.sphinx import versionadded, versionchanged
 from ipywidgets import jsdlink
-from traitlets import Bool, link, observe
+from traitlets import link, observe
+from typing_extensions import Self
 
-import sepal_ui
 from sepal_ui import color
-from sepal_ui.frontend.resize_trigger import rt
+from sepal_ui.frontend.resize_trigger import ResizeTrigger, rt
+from sepal_ui.frontend.styles import get_theme
 from sepal_ui.message import ms
+from sepal_ui.model import Model
 from sepal_ui.scripts import utils as su
 from sepal_ui.sepalwidgets.alert import Banner
 from sepal_ui.sepalwidgets.sepalwidget import SepalWidget
+from sepal_ui.translator import Translator
 
 __all__ = [
     "AppBar",
@@ -28,30 +47,226 @@ __all__ = [
 ]
 
 
+class LocaleSelect(v.Menu, SepalWidget):
+
+    COUNTRIES: pd.DataFrame = pd.read_parquet(
+        Path(__file__).parents[1] / "data" / "locale.parquet"
+    )
+    "the country list as a df. columns [code, name, flag]"
+
+    FLAG: str = "https://flagcdn.com/{}.svg"
+    "the url of the svg flag images"
+
+    ATTR: Dict[str, str] = {
+        "src": "https://flagcdn.com/gb.svg",
+        "width": "30",
+        "alt": "en-UK",
+    }
+    "the default flag parameter, default to english"
+
+    btn: Optional[v.Btn] = None
+    "the btn to click when changing language"
+
+    language_list: Optional[v.List] = None
+    "the list of countries with their flag,name in english, and ISO code"
+
+    def __init__(self, translator: Optional[Translator] = None, **kwargs) -> None:
+        """
+        A language selector for sepal-ui based application.
+
+        It displays the currently requested language (not the one used by the translator).
+        When value is changed, the sepal-ui config file is updated. It is designed to be used in a AppBar component.
+
+        .. warning:: as the component is a v.Menu to get the selected value you need to lisen to "value" instead of "v_model".
+
+        .. versionadded:: 2.7.0
+
+        Args:
+            translator: the translator of the app, to match the used language
+            kwargs (optional): any arguments for a Btn object, children will be override
+        """
+        # extract the available language from the translator
+        # default to only en-US if no translator is set
+        available_locales = (
+            ["en"] if translator is None else translator.available_locales()
+        )
+
+        # extract the language information from the translator
+        # if not set default to english
+        code = "en" if translator is None else translator._target
+        loc = self.COUNTRIES[self.COUNTRIES.code == code].squeeze()
+        attr = {**self.ATTR, "src": self.FLAG.format(loc.flag), "alt": loc.name}
+
+        kwargs.setdefault("small", True)
+        kwargs["v_model"] = False
+        kwargs["v_on"] = "x.on"
+        kwargs["children"] = [v.Html(tag="img", attributes=attr, class_="mr-1"), code]
+        self.btn = v.Btn(**kwargs)
+
+        self.language_list = v.List(
+            dense=True,
+            flat=True,
+            color=color.menu,
+            v_model=True,
+            max_height="300px",
+            style_="overflow: auto; border-radius: 0 0 0 0;",
+            children=[
+                v.ListItemGroup(
+                    children=self._get_country_items(available_locales), v_model=""
+                )
+            ],
+        )
+
+        super().__init__(
+            children=[self.language_list],
+            v_model=False,
+            close_on_content_click=True,
+            v_slots=[{"name": "activator", "variable": "x", "children": self.btn}],
+            value=loc.code,
+        )
+
+        # add js behaviour
+        jsdlink((self.language_list.children[0], "v_model"), (self, "value"))
+        self.language_list.children[0].observe(self._on_locale_select, "v_model")
+
+    def _get_country_items(self, locales: list) -> List[str]:
+        """
+        Get the list of countries as a list of listItem.
+
+        Reduce the list to the available language of the module.
+
+        Args:
+            locales: list of the locales to display
+
+        Returns:
+            the list of contry widget to display in the app
+        """
+        country_list = []
+        filtered_countries = self.COUNTRIES[self.COUNTRIES.code.isin(locales)]
+        for r in filtered_countries.itertuples(index=False):
+
+            attr = {**self.ATTR, "src": self.FLAG.format(r.flag), "alt": r.name}
+
+            children = [
+                v.ListItemAction(children=[v.Html(tag="img", attributes=attr)]),
+                v.ListItemContent(children=[v.ListItemTitle(children=[r.name])]),
+                v.ListItemActionText(children=[r.code]),
+            ]
+
+            country_list.append(v.ListItem(value=r.code, children=children))
+
+        return country_list
+
+    def _on_locale_select(self, change: dict) -> None:
+        """
+        adapt the application to the newly selected language.
+
+        Display the new flag and country code on the widget btn
+        change the value in the config file
+        """
+        # get the line in the locale dataframe
+        loc = self.COUNTRIES[self.COUNTRIES.code == change["new"]].squeeze()
+
+        # change the btn attributes
+        attr = {**self.ATTR, "src": self.FLAG.format(loc.flag), "alt": loc.name}
+        self.btn.children = [
+            v.Html(tag="img", attributes=attr, class_="mr-1"),
+            loc.code,
+        ]
+        self.btn.color = "info"
+
+        # change the paramater file
+        su.set_config("locale", loc.code)
+
+        return
+
+
+class ThemeSelect(v.Btn, SepalWidget):
+
+    THEME_ICONS: dict = {"dark": "fa-solid fa-moon", "light": "fa-solid fa-sun"}
+    "the dictionnry of icons to use for each theme (used as keys)"
+
+    theme: str = "dark"
+    "the current theme of the widget (default to dark)"
+
+    def __init__(self, **kwargs) -> None:
+        """
+        A theme selector for sepal-ui based application.
+
+        It displays the currently requested theme (default to dark).
+        When value is changed, the sepal-ui config file is updated. It is designed to be used in a AppBar component.
+
+        .. versionadded:: 2.7.0
+
+        Args:
+            kwargs: any arguments for a Btn object, children and v_model will be override
+        """
+        # get the current theme name
+        self.theme = get_theme()
+
+        # set the btn parameters
+        kwargs.setdefault("x_small", True)
+        kwargs.setdefault("fab", True)
+        kwargs.setdefault("class_", "ml-2")
+        kwargs["children"] = [v.Icon(children=[self.THEME_ICONS[self.theme]])]
+        kwargs["v_model"] = self.theme
+
+        # create the btn
+        super().__init__(**kwargs)
+
+        # add some js events
+        self.on_event("click", self.toggle_theme)
+
+    def toggle_theme(self, *args) -> None:
+        """
+        Toggle the btn icon from dark to light and adapt the configuration file.
+        """
+        # use a cycle to go through the themes
+        theme_cycle = cycle(self.THEME_ICONS.keys())
+        next(t for t in theme_cycle if t == self.theme)
+        self.theme = next(t for t in theme_cycle)
+
+        # change icon
+        self.color = "info"
+        self.children[0].children = [self.THEME_ICONS[self.theme]]
+
+        # change the paramater file
+        su.set_config("theme", self.theme)
+
+        # trigger other events by changing v_model
+        self.v_model = self.theme
+
+        return
+
+
 class AppBar(v.AppBar, SepalWidget):
-    """
-    Custom AppBar widget with the provided title using the sepal color framework
 
-    Args:
-        title (str, optional): the title of the app
-        translator (sw.Translator, optional): the app translator to pass to the locale selector object
-        kwargs(dict, optional): any parameters from a v.AppBar. If set, 'children' and 'app' will be overwritten.
-    """
+    toogle_button: Optional[v.Btn]
+    "The btn to display or hide the drawer to the user"
 
-    toogle_button = None
-    "v.Btn: The btn to display or hide the drawer to the user"
+    title: Optional[v.ToolbarTitle]
+    "The widget containing the app title"
 
-    title = None
-    "v.ToolBarTitle: the widget containing the app title"
+    locale: Optional[LocaleSelect]
+    "The locale selector of all apps"
 
-    locale = None
-    "sw.LocaleSelect: the locale selector of all apps"
+    theme = Optional[ThemeSelect]
+    "The theme selector of all apps"
 
-    theme = None
-    "sw.ThemeSelect: the theme selector of all apps"
-
-    def __init__(self, title="SEPAL module", translator=None, **kwargs):
+    def __init__(
+        self,
+        title: str = "SEPAL module",
+        translator: Union[None, Translator] = None,
+        **kwargs,
+    ) -> None:
+        """
+        Custom AppBar widget with the provided title using the sepal color framework.
 
+        Args:
+            title: the title of the app
+            translator: the app translator to pass to the locale selector object
+            kwargs (optional): any parameters from a v.AppBar. If set, 'children' and 'app' will be overwritten.
+        """
         self.toggle_button = v.Btn(
             icon=True,
             children=[
@@ -65,9 +280,9 @@ class AppBar(v.AppBar, SepalWidget):
         self.theme = ThemeSelect()
 
         # set the default parameters
-        kwargs["color"] = kwargs.pop("color", color.main)
-        kwargs["class_"] = kwargs.pop("class_", "white--text")
-        kwargs["dense"] = kwargs.pop("dense", True)
+        kwargs.setdefault("color", color.main)
+        kwargs.setdefault("class_", "white--text")
+        kwargs.setdefault("dense", True)
         kwargs["app"] = True
         kwargs["children"] = [
             self.toggle_button,
@@ -79,58 +294,54 @@ class AppBar(v.AppBar, SepalWidget):
 
         super().__init__(**kwargs)
 
-    def set_title(self, title):
+    def set_title(self, title: str) -> Self:
         """
-        Set the title of the appBar
+        Set the title of the appBar.
 
         Args:
-            title (str): the new app title
-
-        Return:
-            self
+            title: the new app title
         """
-
         self.title.children = [title]
 
         return self
 
 
 class DrawerItem(v.ListItem, SepalWidget):
-    """
-    Custom DrawerItem using the user input.
-    If a card is set the drawerItem will trigger the display of all the Tiles in the app that have the same mount_id.
-    If an href is set, the drawer will open the link in a new tab
 
-    Args:
-        title (str): the title of the drawer item
-        icon(str, optional): the full name of a mdi/fa icon
-        card (str, optional): the mount_id of tiles in the app
-        href (str, optional): the absolute link to an external web page
-        kwargs (optional): any parameter from a v.ListItem. If set, '_metadata', 'target', 'link' and 'children' will be overwritten.
-        model (optional): sepalwidget model where is defined the bin_var trait
-        bind_var (optional): required when model is selected. Trait to link with 'alert' self trait parameter
-    """
+    rt: Optional[ResizeTrigger] = None
+    "The trigger to resize maps and other javascript object when jumping from a tile to another"
 
-    rt = None
-    "sw.ResizeTrigger: the trigger to resize maps and other javascript object when jumping from a tile to another"
+    alert: t.Bool = t.Bool(False).tag(sync=True)
+    "Trait to control visibility of an alert in the drawer item"
 
-    alert = Bool(False).tag(sync=True)
-    "Bool: trait to control visibility of an alert in the drawer item"
-
-    alert_badge = None
-    "v.ListItemAction: red circle to display in the drawer"
+    alert_badge: Optional[v.ListItemAction]
+    "red circle to display in the drawer"
 
     def __init__(
         self,
-        title,
-        icon=None,
-        card=None,
-        href=None,
-        model=None,
-        bind_var=None,
+        title: str,
+        icon: str = "",
+        card: str = "",
+        href: str = "",
+        model: Union[Model, None] = None,
+        bind_var: str = "",
         **kwargs,
-    ):
+    ) -> None:
+        """
+        Custom DrawerItem using the user input.
 
+        If a card is set the drawerItem will trigger the display of all the Tiles in the app that have the same mount_id.
+        If an href is set, the drawer will open the link in a new tab.
+
+        Args:
+            title: the title of the drawer item
+            icon: the full name of a mdi/fa icon
+            card: the mount_id of tiles in the app
+            href: the absolute link to an external web page
+            model: sepalwidget model where is defined the bin_var trait
+            bind_var: required when model is selected. Trait to link with 'alert' self trait parameter
+            kwargs (optional): any parameter from a v.ListItem. If set, '_metadata', 'target', 'link' and 'children' will be overwritten.
+        """
         # set the resizetrigger
         self.rt = rt
 
@@ -146,15 +357,15 @@ class DrawerItem(v.ListItem, SepalWidget):
         # set default parameters
         kwargs["link"] = True
         kwargs["children"] = children
-        kwargs["input_value"] = kwargs.pop("input_value", False)
+        kwargs.setdefault("input_value", False)
         if href:
             kwargs["href"] = href  # cannot be set twice anyway
             kwargs["target"] = "_blank"
-            kwargs["_metadata"] = kwargs.pop("_metadata", None)
+            kwargs.setdefault("_metadata", None)
         elif card:
             kwargs["_metadata"] = {"card_id": card}
-            kwargs["href"] = kwargs.pop("href", None)
-            kwargs["target"] = kwargs.pop("target", None)
+            kwargs.setdefault("href", None)
+            kwargs.setdefault("target", None)
 
         # call the constructor
         super().__init__(**kwargs)
@@ -176,9 +387,8 @@ class DrawerItem(v.ListItem, SepalWidget):
             link((model, bind_var), (self, "alert"))
 
     @observe("alert")
-    def add_notif(self, change):
-        """Add a notification alert to drawer"""
-
+    def add_notif(self, change: dict) -> None:
+        """Add a notification alert to drawer."""
         if change["new"]:
             if self.alert_badge not in self.children:
                 new_children = self.children[:]
@@ -189,9 +399,8 @@ class DrawerItem(v.ListItem, SepalWidget):
 
         return
 
-    def remove_notif(self):
-        """Remove notification alert"""
-
+    def remove_notif(self) -> None:
+        """Remove notification alert."""
         if self.alert_badge in self.children:
             new_children = self.children[:]
             new_children.remove(self.alert_badge)
@@ -200,23 +409,20 @@ class DrawerItem(v.ListItem, SepalWidget):
 
         return
 
-    def display_tile(self, tiles):
+    def display_tile(self, tiles: List[v.Card]) -> Self:
         """
         Display the apropriate tiles when the item is clicked.
-        The tile to display will be all tile in the list with the mount_id as the current object
+
+        The tile to display will be all tile in the list with the mount_id as the current object.
 
         Args:
-            tiles ([sw.Tile]) : the list of all the available tiles in the app
-
-        Return:
-            self
+            tiles: the list of all the available tiles in the app
         """
-
         self.on_event("click", partial(self._on_click, tiles=tiles))
 
         return self
 
-    def _on_click(self, widget, event, data, tiles):
+    def _on_click(self, tiles: List[v.Card], *args) -> Self:
 
         for tile in tiles:
             if self._metadata["card_id"] == tile._metadata["mount_id"]:
@@ -237,23 +443,30 @@ class DrawerItem(v.ListItem, SepalWidget):
 
 
 class NavDrawer(v.NavigationDrawer, SepalWidget):
-    """
-    Custom NavDrawer using the different DrawerItems of the user and the sepal color framework.
-    The drawer can include links to the github page of the project for wiki, bugs and repository.
 
-    Args:
-        item ([sw.DrawerItem]): the list of all the drawerItem to display in the drawer. This items should pilote the different tile visibility
-        code (str, optional): the absolute link to the source code
-        wiki (str, optional): the absolute link the the wiki page
-        issue (str, optional): the absolute link to the issue tracker
-        kwargs (optional) any parameter from a v.NavigationDrawer. If set, 'app' and 'children' will be overwritten.
-    """
+    items: List[DrawerItem] = []
+    "the list of all the drawerItem to display in the drawer"
 
-    items = []
-    "list: the list of all the drawerItem to display in the drawer"
+    def __init__(
+        self,
+        items: List[DrawerItem] = [],
+        code: str = "",
+        wiki: str = "",
+        issue: str = "",
+        **kwargs,
+    ) -> None:
+        """
+        Custom NavDrawer using the different DrawerItems of the user.
 
-    def __init__(self, items=[], code=None, wiki=None, issue=None, **kwargs):
+        The drawer can include links to the github page of the project for wiki, bugs and repository.
 
+        Args:
+            items: the list of all the drawerItem to display in the drawer. This items should pilote the different tile visibility
+            code: the absolute link to the source code
+            wiki: the absolute link the the wiki page
+            issue: the absolute link to the issue tracker
+            kwargs (optional) any parameter from a v.NavigationDrawer. If set, 'app' and 'children' will be overwritten.
+        """
         self.items = items
 
         code_link = []
@@ -280,9 +493,9 @@ class NavDrawer(v.NavigationDrawer, SepalWidget):
         ]
 
         # set default parameters
-        kwargs["v_model"] = kwargs.pop("v_model", True)
+        kwargs.setdefault("v_model", True)
         kwargs["app"] = True
-        kwargs["color"] = kwargs.pop("color", color.darker)
+        kwargs.setdefault("color", color.darker)
         kwargs["children"] = children
 
         # call the constructor
@@ -292,30 +505,28 @@ class NavDrawer(v.NavigationDrawer, SepalWidget):
         for i in self.items:
             i.observe(self._on_item_click, "input_value")
 
-    def display_drawer(self, toggleButton):
+    def display_drawer(self, toggleButton: v.Btn) -> Self:
         """
-        Bind the drawer to the app toggleButton
+        Bind the drawer to the app toggleButton.
 
         Args:
-            toggleButton(v.Btn) : the button that activate the drawer
+            toggleButton: the button that activate the drawer
         """
-
         toggleButton.on_event("click", self._on_drawer_click)
 
         return self
 
-    def _on_drawer_click(self, widget, event, data):
+    def _on_drawer_click(self, *args) -> Self:
         """
-        Toggle the drawer visibility
+        Toggle the drawer visibility.
         """
-
         self.v_model = not self.v_model
 
         return self
 
-    def _on_item_click(self, change):
+    def _on_item_click(self, change: dict) -> Self:
         """
-        Deactivate all the other items when on of the is activated
+        Deactivate all the other items when on of the is activated.
         """
         if change["new"] is False:
             return self
@@ -329,22 +540,21 @@ class NavDrawer(v.NavigationDrawer, SepalWidget):
 
 
 class Footer(v.Footer, SepalWidget):
-    """
-    Custom Footer with cuzomizable text.
-    Not yet capable of displaying logos
+    def __init__(self, text: str = "", **kwargs) -> None:
+        """
+        Custom Footer with cuzomizable text.
 
-    Args:
-        text (str, optional): the text to display in the future
-        kwargs (optional): any parameter from a v.Footer. If set ['app', 'children'] will be overwritten.
-    """
-
-    def __init__(self, text="", **kwargs):
+        Not yet capable of displaying logos.
 
+        Args:
+            text: the text to display in the future
+            kwargs (optional): any parameter from a v.Footer. If set ['app', 'children'] will be overwritten.
+        """
         text = text if text != "" else "SEPAL \u00A9 {}".format(datetime.today().year)
 
         # set default parameters
-        kwargs["color"] = kwargs.pop("color", color.main)
-        kwargs["class_"] = kwargs.pop("class_", "white--text")
+        kwargs.setdefault("color", color.main)
+        kwargs.setdefault("class_", "white--text")
         kwargs["app"] = True
         kwargs["children"] = [text]
 
@@ -353,47 +563,47 @@ class Footer(v.Footer, SepalWidget):
 
 
 class App(v.App, SepalWidget):
-    """
-    Custom App display with the tiles created by the user using the sepal color framework.
-    Display false appBar if not filled. Navdrawer is fully optionnal.
-    The drawerItem will be linked to the app tile and they will be able to control their display
-    If the navdrawer exist, it will be linked to the appbar togglebtn
 
-    Args:
-        tiles ([sw.Tile]): the tiles of the app
-        appBar (sw.AppBar, optional): the appBar of the application
-        footer (sw.Footer, optional): the footer of the application
-        navDrawer (sw.NavDrawer, optional): the navdrawer of the application
-        translator (sw.Translator, optional): the translator of the app to display language informations
-        kwargs (optional) any parameter from a v.App. If set, 'children' will be overwritten.
-    """
+    tiles: List[v.Card] = []
+    "the tiles of the app"
 
-    tiles = []
-    "list: the tiles of the app"
+    appBar: Optional[AppBar] = None
+    "the AppBar of the application"
 
-    appBar = None
-    "sw.AppBar: the AppBar of the application"
+    footer: Optional[Footer] = None
+    "the footer of the application"
 
-    footer = None
-    "sw.Footer: the footer of the application"
+    navDrawer: Optional[NavDrawer] = None
+    "the navdrawer of the application"
 
-    navDrawer = None
-    "sw.NavDrawer: the navdrawer of the application"
-
-    content = None
-    "v.Content: the tiles organized in a fluid container"
+    content: Optional[v.Content] = None
+    "the tiles organized in a fluid container"
 
     def __init__(
         self,
-        tiles=[""],
-        appBar=None,
-        footer=None,
-        navDrawer=None,
-        translator=None,
+        tiles: List[v.Card] = [],
+        appBar: Optional[AppBar] = None,
+        footer: Optional[Footer] = None,
+        navDrawer: Optional[NavDrawer] = None,
+        translator: Optional[Translator] = None,
         **kwargs,
-    ):
+    ) -> None:
+        """
+        Custom App display with the tiles created by the user using the sepal color framework.
 
-        self.tiles = None if tiles == [""] else tiles
+        Display false appBar if not filled. Navdrawer is fully optionnal.
+        The drawerItem will be linked to the app tile and they will be able to control their display
+        If the navdrawer exist, it will be linked to the appbar togglebtn.
+
+        Args:
+            tiles: the tiles of the app
+            appBar: the appBar of the application
+            footer: the footer of the application
+            navDrawer: the navdrawer of the application
+            translator: the translator of the app to display language informations
+            kwargs (optional) any parameter from a v.App. If set, 'children' will be overwritten.
+        """
+        self.tiles = tiles
 
         app_children = []
 
@@ -433,7 +643,7 @@ class App(v.App, SepalWidget):
         bg = v.Overlay(color=color.bg, opacity=1, style_="transition:unset", z_index=-1)
 
         # set default parameters
-        kwargs["v_model"] = kwargs.pop("v_model", None)
+        kwargs.setdefault("v_model", None)
         kwargs["children"] = [bg, *app_children]
 
         # call the constructor
@@ -451,15 +661,12 @@ class App(v.App, SepalWidget):
         self.appBar.locale.observe(self._locale_info, "value")
         self.appBar.theme.observe(self._theme_info, "v_model")
 
-    def show_tile(self, name):
+    def show_tile(self, name: str) -> Self:
         """
-        Select the tile to display when the app is launched
+        Select the tile to display when the app is launched.
 
         Args:
-            name (str): the mount-id of the tile(s) to display
-
-        Return:
-            self
+            name: the mount-id of the tile(s) to display
         """
         # show the tile
         for tile in self.tiles:
@@ -479,18 +686,26 @@ class App(v.App, SepalWidget):
 
     @versionadded(version="2.4.1", reason="New end user interaction method")
     @versionchanged(version="2.7.1", reason="new id\_ and persistent parameters")
-    def add_banner(self, msg="", type_="info", id_=None, persistent=True, **kwargs):
-        """
-        Display an snackbar object on top of the app to communicate development information to end user (release date, known issues, beta version). The alert is dissmisable and prominent.
+    def add_banner(
+        self,
+        msg: str = "",
+        type_: str = "info",
+        id_=None,
+        persistent: bool = True,
+        **kwargs,
+    ) -> Self:
+        r"""
+        Display a snackbar object on top of the app.
+
+        Used to communicate development information to end user (release date, known issues, beta version). The alert is dissmisable and prominent.
 
         Args:
-            *args: all required sw.Banner arguments.
+            msg: Message to display in application banner. default to nothing
+            type\_: Used to display an appropiate banner color. fallback to "info".
+            id_: unique banner identificator.
+            persistent: Whether to close automatically based on the lenght of message (False) or make it indefinitely open (True). Overridden if timeout duration is set.
             **kwargs: any arguments of the sw.Banner constructor. if set, 'children' will be overwritten.
-
-        Return:
-            self
         """
-
         # the Banner was previously an Alert. for compatibility we accept the type parameter
         type_ = kwargs.pop("type", type_)
 
@@ -522,28 +737,27 @@ class App(v.App, SepalWidget):
 
         return self
 
-    def _locale_info(self, change):
-        """display information about the locale change"""
-
+    def _locale_info(self, change: dict) -> None:
+        """display information about the locale change."""
         if change["new"] != "":
             msg = ms.locale.change.format(change["new"])
             self.add_banner(msg)
 
         return
 
-    def _theme_info(self, change):
-        """display information about the theme change"""
-
+    def _theme_info(self, change: dict) -> None:
+        """display information about the theme change."""
         if change["new"] != "":
             msg = ms.theme.change.format(change["new"])
             self.add_banner(msg)
 
         return
 
-    def _remove_banner(self, change):
+    def _remove_banner(self, change: dict) -> None:
         """
-        Adapt the banner display so that the first one is the oly one shown displaying the number of other banner in the queue
-        I'm force to create a function as lambda method cannot do assignments before python 3.9.
+        remove banner and adapt display of the others.
+
+        Adapt the banner display so that the first one is the only one shown displaying the number of other banner in the queue
         """
         if change["new"] is False:
 
@@ -566,184 +780,3 @@ class App(v.App, SepalWidget):
             self.content.children = banner_list + children
 
         return
-
-
-class LocaleSelect(v.Menu, SepalWidget):
-    """
-    An language selector for sepal-ui based application.
-
-    It displays the currently requested language (not the one used by the translator).
-    When value is changed, the sepal-ui config file is updated. It is designed to be used in a AppBar component.
-
-    .. warning:: as the component is a v.Menu to get the selected value you need to lisen to "value" instead of "v_model".
-
-    .. versionadded:: 2.7.0
-
-
-    Args:
-        translator (sw.Translator, optional): the translator of the app, to match the used language
-        kwargs (dict, optional): any arguments for a Btn object, children will be override
-    """
-
-    COUNTRIES = pd.read_csv(Path(__file__).parents[1] / "scripts" / "locale.csv")
-    "pandas.DataFrame: the country list as a df. columns [code, name, flag]"
-
-    FLAG = "https://flagcdn.com/{}.svg"
-    "str: the url of the svg flag images"
-
-    ATTR = {"src": "https://flagcdn.com/gb.svg", "width": "30", "alt": "en-UK"}
-    "dict: the default flag parameter, default to english"
-
-    btn = None
-    "v.Btn: the btn to click when changing language"
-
-    language_list = None
-    "v.List: the list of countries with their flag,name in english, and ISO code"
-
-    def __init__(self, translator=None, **kwargs):
-
-        # extract the available language from the translator
-        # default to only en-US if no translator is set
-        available_locales = (
-            ["en"] if translator is None else translator.available_locales()
-        )
-
-        # extract the language information from the translator
-        # if not set default to english
-        code = "en" if translator is None else translator._target
-        loc = self.COUNTRIES[self.COUNTRIES.code == code].squeeze()
-        attr = {**self.ATTR, "src": self.FLAG.format(loc.flag), "alt": loc.name}
-
-        kwargs["small"] = kwargs.pop("small", True)
-        kwargs["v_model"] = False
-        kwargs["v_on"] = "x.on"
-        kwargs["children"] = [v.Html(tag="img", attributes=attr, class_="mr-1"), code]
-        self.btn = v.Btn(**kwargs)
-
-        self.language_list = v.List(
-            dense=True,
-            flat=True,
-            color=color.menu,
-            v_model=True,
-            max_height="300px",
-            style_="overflow: auto; border-radius: 0 0 0 0;",
-            children=[
-                v.ListItemGroup(
-                    children=self._get_country_items(available_locales), v_model=""
-                )
-            ],
-        )
-
-        super().__init__(
-            children=[self.language_list],
-            v_model=False,
-            close_on_content_click=True,
-            v_slots=[{"name": "activator", "variable": "x", "children": self.btn}],
-            value=loc.code,
-        )
-
-        # add js behaviour
-        jsdlink((self.language_list.children[0], "v_model"), (self, "value"))
-        self.language_list.children[0].observe(self._on_locale_select, "v_model")
-
-    def _get_country_items(self, locales):
-        """get the list of countries as a list of listItem. reduce the list to the available language of the module"""
-
-        country_list = []
-        filtered_countries = self.COUNTRIES[self.COUNTRIES.code.isin(locales)]
-        for r in filtered_countries.itertuples(index=False):
-
-            attr = {**self.ATTR, "src": self.FLAG.format(r.flag), "alt": r.name}
-
-            children = [
-                v.ListItemAction(children=[v.Html(tag="img", attributes=attr)]),
-                v.ListItemContent(children=[v.ListItemTitle(children=[r.name])]),
-                v.ListItemActionText(children=[r.code]),
-            ]
-
-            country_list.append(v.ListItem(value=r.code, children=children))
-
-        return country_list
-
-    def _on_locale_select(self, change):
-        """
-        adapt the application to the newly selected language
-
-        Display the new flag and country code on the widget btn
-        change the value in the config file
-        """
-
-        # get the line in the locale dataframe
-        loc = self.COUNTRIES[self.COUNTRIES.code == change["new"]].squeeze()
-
-        # change the btn attributes
-        attr = {**self.ATTR, "src": self.FLAG.format(loc.flag), "alt": loc.name}
-        self.btn.children = [
-            v.Html(tag="img", attributes=attr, class_="mr-1"),
-            loc.code,
-        ]
-        self.btn.color = "info"
-
-        # change the paramater file
-        su.set_config("locale", loc.code)
-
-        return
-
-
-class ThemeSelect(v.Btn, SepalWidget):
-    """
-    A theme selector for sepal-ui based application.
-
-    It displays the currently requested theme (default to dark).
-    When value is changed, the sepal-ui config file is updated. It is designed to be used in a AppBar component.
-
-    .. versionadded:: 2.7.0
-
-    Args:
-        kwargs (dict, optional): any arguments for a Btn object, children and v_model will be override
-    """
-
-    THEME_ICONS = {"dark": "fa-solid fa-moon", "light": "fa-solid fa-sun"}
-    "dict: the dictionnry of icons to use for each theme (used as keys)"
-
-    theme = "dark"
-    "str: the current theme of the widget (default to dark)"
-
-    def __init__(self, **kwargs):
-
-        # get the current theme name
-        self.theme = sepal_ui.get_theme()
-
-        # set the btn parameters
-        kwargs["x_small"] = kwargs.pop("x_small", True)
-        kwargs["fab"] = kwargs.pop("fab", True)
-        kwargs["class_"] = kwargs.pop("class_", "ml-2")
-        kwargs["children"] = [v.Icon(children=[self.THEME_ICONS[self.theme]])]
-        kwargs["v_model"] = self.theme
-
-        # create the btn
-        super().__init__(**kwargs)
-
-        # add some js events
-        self.on_event("click", self.toggle_theme)
-
-    def toggle_theme(self, widget, event, data):
-        """
-        toggle the btn icon from dark to light and adapt the configuration file at the same time
-        """
-        # use a cycle to go through the themes
-        theme_cycle = cycle(self.THEME_ICONS.keys())
-        next(t for t in theme_cycle if t == self.theme)
-        self.theme = next(t for t in theme_cycle)
-
-        # change icon
-        self.color = "info"
-        self.children[0].children = [self.THEME_ICONS[self.theme]]
-
-        # change the paramater file
-        su.set_config("theme", self.theme)
-
-        # trigger other events by changing v_model
-        self.v_model = self.theme
-
-        return
